;********************************************************** UPDATED 26/05/08 for both NoVRAM and NoV-32
; ---- Block for reading Address from DATA and ISA line, and send it to serial RAM (WRITE CYCLE) ***********************************************************

; The serial interface sequence will look like this (Address write portion):
;Phi# |    14   |    15   |    16   |    17   |    18   |    19   |    20   |    21   |    22   |    23   |    24   |    25   |    26   |    27   |    28   |    29   |    30  |
;CLK __/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\____________/\__/\__/\__/\__/\__/\__/\__/\__/\_______/\_______/\__/\__/\__/\__/\__/\__/\_____
;
;DAT ___/\__/\_____/\_____________________________________________XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX_____XXXXX_____XXXXX_____XXXXXXXXXXXXXXXXXXXXXXXXX______________
;       ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^              ^    ^    ^    ^    ^    ^    ^    ^    ^         ^         ^    ^    ^    ^    ^    ^    ^   
;     STOP START  1    0    1    0    0    0    0    0   ACK  "0"            AL0  AL1  AL2  AL3  AL4  AL5  AL6   ACK AL7       AH0       AH1  AH2  AH3  AH4  AH5  "0"  ACK  
;                |----SLAVE ID----|  |-DEVICE AD-| WRITE     | a15           a14  a13  a12  a11  a10  a09  a08       a07       a06       a05  a04  a03  a02  a01  a00   See NOTE
; NOTE: The address sent to the RAM is obtained from ISA or DATA depending if it's a WRITE s&x or not.
;       After the last ACK on pulse #30 we'll check if the received address is in our RAM and if it's a WRITE CYCLE, see the rest of the timing chart for Data Write in Phase #30

	wtre	NUTPHI1		; 14
;--------------------------------------------------------------------- (14)
	btfsc	NUTDATA
	bsf	adrlw,0		; Read DATA line address low, bit 0

	bsf	RAMCLK		; Set SCL high
	nop			; CLK ___/
	nop
	bsf	RAMDAT		; Set SDT high (STOP condition)
	call	s4nop		; DAT _______/
	bcf	RAMDAT		; SDT to "0" (Start sequence)
	nop			; DAT \________
	nop
	bcf 	RAMCLK		; Sets SCL low
				; CLK \___

	wthi	NUTPHI1		; 15
;--------------------------------------------------------------------- (24)	
	btfsc	NUTDATA
	bsf	adrlw,1		; Read DATA line address low, bit 1

	bsf	RAMDAT		; SDT to "1" (Start slave address "B'10100000")
	call	ferclk
;------------------- 
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100000" 2nd bit)
	call	ferclk

	wthi	NUTPHI1		; 16
;---------------------------------------------------------------------- (26)
	btfsc	NUTDATA
	bsf	adrlw,2		; Read DATA line address low, bit 2

	bsf	RAMDAT		; SDT to "1" (Slave address "B'10100000" 3rd bit)
	btfsc	NUTISA
	bsf	addrl,0		; Read ISA line address low, bit 0
	call	ferclk
;------------------- 
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100000" 4th bit)
	call	ferclk

	wthi	NUTPHI1		; 17
;----------------------------------------------------------------------- (26)
	btfsc	NUTDATA
	bsf	adrlw,3		; Read DATA line address low, bit 3

	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100000" 5th bit)
	btfsc	NUTISA
	bsf	addrl,1		; Read ISA line address low, bit 1
	call	ferclk
;------------------- 
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100000" 6th bit)
	call	ferclk

	wthi	NUTPHI1		; 18
;-------------------------------------------------------------------------- (26)
	btfsc	NUTDATA
	bsf	adrlw,4		; Read DATA line address low, bit 4

	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100000" 7th bit)
	btfsc	NUTISA
	bsf	addrl,2		; Read ISA line address low, bit 2
	call	ferclk
;------------------- 
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100000" 8th bit)
	call	ferclk

	wthi	NUTPHI1		; 19
;------------------------------------------------------------------------------- (27)
	btfsc	NUTDATA
	bsf	adrlw,5		; Read DATA line address low, bit 5

	bsf	DATDIR		; Set RAMDAT as Input for ACK
	btfsc	NUTISA
	bsf	addrl,3		; Read ISA line address low, bit 3
	call	ferclk
	bcf	DATDIR		; Set RAMDAT as output again.
;------------------- 
	bcf	RAMDAT		; SDT to "0", first Address to RAM (A15) always "0"
	call	ferclk

	wthi	NUTPHI1		; 20
;------------------------------------------------------------------------------- (8)
	btfsc	NUTDATA
	bsf	adrlw,6		; Read DATA line address low, bit 6

	btfsc	NUTISA
	bsf	addrl,4		; Read ISA line address low, bit 4

	clrf	FSR0L
	bsf	FSR0L,6		; Points to Low Address ISA
	btfsc	adrsource,0	; Selects address form ISA...
	bsf	FSR0L,2		; ... or from DATA

	wthi	NUTPHI1		; 21
;------------------------------------------------------------------------------- (30)
	btfsc	NUTDATA
	bsf	adrlw,7		; Read DATA line address low, bit 7

	bcf	RAMDAT
	btfsc	INDF0,0
	bsf	RAMDAT		; Write SDT to address L,0 sent as A14
	btfsc	NUTISA
	bsf	addrl,5		; Read ISA line address low, bit 5
	call	ferclk
;------------------- 
	bcf	RAMDAT
	btfsc	INDF0,1
	bsf	RAMDAT		; Write SDT to address L,1 sent as A13
	call	ferclk

	wthi	NUTPHI1		; 22
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	adrhg,0		; Read DATA line address high, bit 0

	bcf	RAMDAT
	btfsc	INDF0,2
	bsf	RAMDAT		; Write SDT to address L,2 sent as A12
	btfsc	NUTISA
	bsf	addrl,6		; Read ISA line address low, bit 6
	call	ferclk
;-------------------
	bcf	RAMDAT
	btfsc	INDF0,3
	bsf	RAMDAT		; Write SDT to address L,3 sent as A11
	call	ferclk

	wthi	NUTPHI1		; 23
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	adrhg,1		; Read DATA line address high, bit 1

	bcf	RAMDAT
	btfsc	INDF0,4
	bsf	RAMDAT		; Write SDT to address L,4 sent as A10
	btfsc	NUTISA
	bsf	addrl,7		; Read ISA line address low, bit 7
	call	ferclk
;-------------------
	bcf	RAMDAT
	btfsc	INDF0,5
	bsf	RAMDAT		; Write SDT to address L,5 sent as A9
	call	ferclk

	wthi	NUTPHI1		; 24
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	adrhg,2		; Read DATA line address high, bit 2

	bcf	RAMDAT
	btfsc	INDF0,6
	bsf	RAMDAT		; Write SDT to address L,6 sent as A8
	btfsc	NUTISA
	bsf	addru,0		; Read ISA line address high, bit 0
	call	ferclk
;-------------------
	bsf	DATDIR		; Set RAMDAT as Input for ACK
	call	ferclk
	bcf	DATDIR		; Set RAMDAT as output again.

	wthi	NUTPHI1		; 25
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	adrhg,3		; Read DATA line address high, bit 3

	bcf	RAMDAT
	btfsc	INDF0,7
	bsf	RAMDAT		; Write SDT to address L,7 sent as A7

	btfsc	NUTISA
	bsf	addru,1		; Read ISA line address high, bit 1
	call	ferclk

	incf	FSR0L		; Points to high address

	wthi	NUTPHI1		; 26
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	adrhg,4		; Read DATA line address high, bit 4

	bcf	RAMDAT
	btfsc	INDF0,0
	bsf	RAMDAT		; Write SDT to address H,0 sent as A6
	btfsc	NUTISA
	bsf	addru,2		; Read ISA line address high, bit 2
	call	ferclk

	wthi	NUTPHI1		; 27
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	adrhg,5		; Read DATA line address high, bit 5

	bcf	RAMDAT
	btfsc	INDF0,1
	bsf	RAMDAT		; Write SDT to address H,1 sent as A5
	btfsc	NUTISA
	bsf	addru,3		; Read ISA line address high, bit 3
	call	ferclk
;-------------------
	bcf	RAMDAT
	btfsc	INDF0,2
	bsf	RAMDAT		; Write SDT to address H,2 sent as A4
	call	ferclk

	wthi	NUTPHI1		; 28
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	adrhg,6		; Read DATA line address high, bit 6

	bcf	RAMDAT
	btfsc	INDF0,3
	bsf	RAMDAT		; Write SDT to address H,3 sent as A3
	btfsc	NUTISA
	bsf	addru,4		; Read ISA line address high, bit 4
	call	ferclk
;-------------------
	bcf	RAMDAT
	btfsc	INDF0,4
	bsf	RAMDAT		; Write SDT to address H,4 sent as A2
	call	ferclk

	wthi	NUTPHI1		; 29
;------------------------------------------------------------------------------- (29)
	btfsc	NUTISA
	bsf	addru,5		; Read ISA line address high, bit 5

	bcf	RAMDAT
	btfsc	INDF0,5
	bsf	RAMDAT		; Write SDT to address H,5 sent as A1
	btfsc	NUTDATA
	bsf	adrhg,7		; Read DATA line address high, bit 7
	call	ferclk
;------------------- 
	nop
	bcf	RAMDAT		; SDT to "0", last Address to RAM (A0) always "0"
	call	ferclk

	wthi	NUTPHI1		; 30
;-------------------------------------------------------------------------------
	btfsc	NUTISA
	bsf	addru,6		; Read ISA line address high, bit 6

	bsf	DATDIR		; Set RAMDAT as Input for ACK
	call	ferclk
	bcf	DATDIR		; Set RAMDAT as output again.

	btfss	adrsource,0	; If itsn't a RAM write...
	goto	ramrd		; it may be a RAM/ROM read

	movf	adrhg,W		; It is a RAM WRITE CYCLE
	andlw	0xF0		; use the upper four bits ...
	swapf	WREG
	movwf	FSR0L		; as a pointer to the map table
	movf	INDF0,W
	btfss	WREG,0		; If map is B'XXXXXXX1 then is our RAM
	bra	notouram
	btfss	WREG,1		; Write protection flag, if B'XXXXXX1X it's protected
	bra	ouramwr

notouram			; Send a STOP sequence.

	wthi	NUTPHI1		; 31'
;-------------------------------------------------------------------------------
	bcf	RAMDAT	
	btfsc	NUTISA
	bsf	addru,7		; Read ISA line address high, bit 7
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	call	s4nop		
	bsf	RAMDAT		; 4 (STOP)
	nop			; and 5, go down...
	nop
	bcf	RAMCLK
	nop
	bcf	RAMDAT
	movf	adrhg,W		; Gets address upper byte
	sublw	0x41		; Is it H'41?
	bnz	chekrom		; If it ist'n go to checkrom
	movf	adrlw,W		; Gets address lower byte
	bnz	chekrom		; If it ist'n go to checkrom
	movf	dtl8,W		; Gets RAM map byte	
	movwf	rammap00	; place it into Ram map register
	movf	dth2,W		; Gets RAM map upper 2bits	
	movwf	rammap01	; place it into Ram map register
chekrom
	goto	checkrom

ouramwr

	wthi	NUTPHI1		; 31
;-------------------------------------------------------------------------------
	bcf	RAMDAT
	btfsc	dtl8,0
	bsf	RAMDAT		; SDT to dtl8,0 
	btfsc	NUTISA
	bsf	addru,7		; ISA line address high, bit 7
	call	ferclk
;------------------- 
	bcf	RAMDAT
	btfsc	dtl8,1
	bsf	RAMDAT		; SDT to dtl8,1 
	call	ferclk

	wthi	NUTPHI1		; 32
;-------------------------------------------------------------------------------
	bcf	RAMDAT
	btfsc	dtl8,2
	bsf	RAMDAT		; SDT to dtl8,2
	call	ferclk
;------------------- 
	bcf	RAMDAT
	btfsc	dtl8,3
	bsf	RAMDAT		; SDT to dtl8,3 
	call	ferclk

	wthi	NUTPHI1		; 33
;-------------------------------------------------------------------------------
	bcf	RAMDAT
	btfsc	dtl8,4
	bsf	RAMDAT		; SDT to dtl8,4
	call	ferclk
;------------------- 
	bcf	RAMDAT
	btfsc	dtl8,5
	bsf	RAMDAT		; SDT to dtl8,5 
	call	ferclk

	wthi	NUTPHI1		; 34
;-------------------------------------------------------------------------------
	bcf	RAMDAT
	btfsc	dtl8,6
	bsf	RAMDAT		; SDT to dtl8,6
	call	ferclk
;------------------- 
	bcf	RAMDAT
	btfsc	dtl8,7
	bsf	RAMDAT		; SDT to dtl8,7 
	call	ferclk

	wthi	NUTPHI1		; 35
;-------------------------------------------------------------------------------
	bcf	RAMDAT
	nop
	bsf	DATDIR		; Set RAMDAT as Input for ACK
	call	ferclk
	bcf	DATDIR		; Set RAMDAT as output again.

	wthi	NUTPHI1		; 36
;-------------------------------------------------------------------------------
	bcf	RAMDAT
	btfsc	dth2,0
	bsf	RAMDAT		; SDT to dth2,0
	call	ferclk
;------------------- 
	bcf	RAMDAT
	btfsc	dth2,1
	bsf	RAMDAT		; SDT to dth2,1
	call	ferclk 

	wthi	NUTPHI1		; 37
;-------------------------------------------------------------------------------
	bcf	RAMDAT		; Set dth2,2 to dth2,7 = "0"
	call	ferclk
;------------------- 
	call	s4nop		; dth2,3
	call	ferclk

	wthi	NUTPHI1		; 38
;-------------------------------------------------------------------------------
	call	ferclk		; dth2,4
;------------------- 
	call	s4nop		; dth2,5
	call	ferclk	

	wthi	NUTPHI1		; 39
;-------------------------------------------------------------------------------
	call	ferclk		; dth2,6
;------------------- 
	call	s4nop		; dth2,7
	call	ferclk

	wthi	NUTPHI1		; 40
;-------------------------------------------------------------------------------
	bsf	DATDIR		; Set RAMDAT as Input for ACK
	call	ferclk
	bcf	DATDIR		; Set RAMDAT as output again.

	bcf	RAMDAT		; Send a STOP sequence.
	nop
	nop
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	call	s4nop		
	bsf	RAMDAT		; and 5, go down...
	call	s4nop
	bcf	RAMCLK
	bcf	RAMDAT

	wthi	NUTPHI1		; 41
;-------------------------------------------------------------------------------
	goto	endwrite	

