; ********************************** Updated as of 13/07/08
; ******************** NoV-32 project Mar 27, 2004 ***********************
;
;Prototyping NoVRAM ver. 07 (This module will emulate HEPAX + 16K RAM + 8K ROM)
;
;ver 01: BankSwitch ok in RUN mode, stuck on bank 3 in PRGM mode.
;
;ver 02: BS problem solved, RAM requires intialization, extra-pages don't appear in HEPAX 002.
;
;ver 03: RAM initialized (manually), Extra-pages BS added so they also appear in HEPAX 002 catalog.
;        Fully operative. Saved as NoVRAM42.asm 
;
;ver 04: Disabling ROM (ISA address) reading if WRITE s&x cycle. This is to verify serial memory
;	  compatibility.
;        It seemed to work flawlessly, still some test to run, 12/3 20:40
;
;ver 05: I'll start the relocation (only RAM, according to B4 line) test here it's 14/3 02:49... 
;        Long way ahead... (note this is ver 05 for NoVRAM42!!) Achieved!! 15/05 00:45
;
;ver 06: Relocation of ROM (the 1st big challenge!! 2nd will be serial FeRAM interface... Wow!!!)
;	 This is the saved version that worked fine!! Next test file is NoVRAM43.asm
;
;ver 07: Serial RAM interface, back to 18LF252. Code fully redesigned.
;        First we'll try only with David Assembler. Whith RAM fixed at 8 to B pages
;
;Ver 07b:Just without the Phi waiting loops
;
;Ver 08b: Working HEPAX with serial non volatile RAM FM24C256
;
;Ver 08c: Implementing RAMTOG (H'1F0) and trying to avoid the 2nd (and unwanted) re-location after plug-in.
;	  RAMWRITE.ASM also modified to RAMWR08c.asm
; 	  The RAMTOG function has been implemented, also want to reduce the overall power drain...
;	  Power reduced at STAND-BY and SLEEP (<35uA and <6uA respectively)
;	  The nasty "32nd PWO fall reset" has been fixed (was caused by stack overflow)
;	  The Advanced HEPAX module is now fully replycated ;-)
;
;Ver 08d: Trying to shrink the code... FeRAM CLK as subroutine...
;	  As for 9 Apr. '04 the code has been reduced... but still is 224 instructions over the H'03FF limit...
;	  ... reaching H'04DF... too bad :-(
;
;Ver 08e: New concept, trying to get address in a single subroutine instead of two...
;	  Address reading subroutine has been renamed to ADDRS08e
;
;Ver 08k: Power to FeRAM supplied by PORTA pin, supressed the B4 mapping feature, as it has no real utility.
;	  This greatly simplifies de HW design, as only two resistors are necessary now!!
;
;ver 08m: Auto-OFF bug is fixed... Geir will enjoy it a lot!! :-)
;
;-Dec. 20th 2011, mods to fix faulty operation with HP-41CL


	list 	  P = 18f252	; set processor type
	list 	  n = 0		; supress page breaks in list file
	#include <p18f252.inc>	; Processor Include file
	list			; turn listing on

; -*- Configuration constants

	__config 0x300001,0x26 ; -*- Enables PLL oscillator mode
	__config 0x300002,0x0d ; -*- Prevents Brown out Reset
	__config 0x300003,0x0e ; -*- Disables Watch Dog Timer
	__config 0x300006,0x80 ; -*- Disables Low Voltage Programming
			       ;     and avoids Stack Overflow reset

; -*- Config Signature **************************************************

#include	"sgnaturH.asm" ; -*- Writes EEPROM signature data.
	
; -*- End NoVRAM Config & Signature *************************************

; We emulate bank-switched HP41 ROMs using a PIC 18LF252.

; The 18LF252 has 32K of memory.  We allocate it thus:
; 0000 - 07ff (2K) available for code
; 0800 - 1fff (6K) packed high-order bits of six 4K ROMs
; 2000 - 7fff (24K) low-order bits of six 4K ROMs

; Definitions of I/O lines **********************************************

#define	NUTDATA	PORTB,7
#define	NUTPHI2	PORTB,6 
#define	NUTSYNC	PORTB,5 
#define	NUTISA	PORTB,4
#define	NUTPHI1	PORTB,2

#define	NUTPWO	PORTB,0
 
;#define	OUTISA	PORTB,4
#define	OUTISA	TRISB,4

; End I/O definitions ***************************************************

; FeRAM interface definitions *******************************************

#define	RAMDAT	PORTA,0 
#define	RAMCLK	PORTA,1
#define RAMPOW1	PORTA,2
#define RAMPOW	PORTA,3
#define DATDIR	TRISA,0

; End FeRAM interface definitions ***************************************

; Define where to map each page/bank, and variables *********************

		CBLOCK 0x00

	rom0b1map
	rom1b1map
	rom2b1map
	rom3b1map
	rom4b1map
	rom5b1map
	rom6b1map
	rom7b1map
	rom8b1map
	rom9b1map
	romab1map
	rombb1map
	romcb1map
	romdb1map
	romeb1map
	romfb1map
	rom0b3map
	rom1b3map
	rom2b3map
	rom3b3map
	rom4b3map
	rom5b3map
	rom6b3map
	rom7b3map
	rom8b3map
	rom9b3map
	romab3map
	rombb3map
	romcb3map
	romdb3map
	romeb3map
	romfb3map
	rom0b2map
	rom1b2map
	rom2b2map
	rom3b2map
	rom4b2map
	rom5b2map
	rom6b2map
	rom7b2map
	rom8b2map
	rom9b2map
	romab2map
	rombb2map
	romcb2map
	romdb2map
	romeb2map
	romfb2map
	rom0b4map
	rom1b4map
	rom2b4map
	rom3b4map
	rom4b4map
	rom5b4map
	rom6b4map
	rom7b4map
	rom8b4map
	rom9b4map
	romab4map
	rombb4map
	romcb4map
	romdb4map
	romeb4map
	romfb4map

	addrl		; lower 8 bits of ISA address	
	addru		; upper 8 bits of ISA address
	opl8		; low eight bits of fetched word
	oph2		; high two bits of fetched word
	adrlw		; DATA line Higer byte received address
	adrhg		; (You can guess for sure)
	dtl8		; DATA line lower 8 bits of data
	dth2		; (Do we bet? ;-)
	enromoff       	; 0x00, 0x20, 0x10 or 0x30 indicating ENBANK1, 2, 3, or 4
	adrsource	; bit0 = 0 address from ISA, bit0 = 1 address from DATA
	locpend		; ROM re-location pending Bit 0=1, re-location done Bit 0=0
	rammap00
	rammap01
;	DAL		; DumpAddressLow RAM
;	DAH		; DumpAddressHigh RAM
;	DDL8		; DumpDataLow8
;	DDH2		; DumpDataHigh2
;	DASL		; DumpAddressLowShifted Flash (Every RAM Word requires TWO Flash bytes!)
;	DASH		; DumpAddressHighShifted Flash
;	LOOPCOUNT	; Dump Loop count 16
;	COUNTER_HI	; Dump 64 groups of 8 bytes each
;	COUNTER		; Dump bytes 1 to 8
		ENDC

; End of variables and mapping/address definitions *************************

; Macros definition ********************************************************
		
wthi	macro	nutport,nutsignal	; loops until signal is hi
	local	ll
ll	btfss	nutport,nutsignal
	bra		ll
	endm

wtlo	macro	nutport,nutsignal	; loops until signal is lo
	local 	ll
ll	btfsc	nutport,nutsignal
	bra		ll
	endm

wtre	macro	nutport,nutsignal	; waits for rising edge
	wtlo	nutport,nutsignal
	wthi	nutport,nutsignal
	endm

wtfe	macro	nutport,nutsignal	; waits for falling edge
	wthi	nutport,nutsignal
	wtlo	nutport,nutsignal
	endm

; End of macros definition *********************************************

;************************* CODE BEGINNING ******************************

; Reset vector

rst	org	0x000000
	clrf	rammap00
	clrf	rammap01
	goto	begin

; Interrupt 0 (high priority) vector

itrrh	org 	0x000008	; Detected PWO falling edge
	clrf	STKPTR
	bra	start		; Go to SLEEP and waits for PWO rising edge

begin	org	0x00000C
		
	movlw	0x07
	movwf	ADCON1		; Port A is digital I/O
	clrf	PORTA		; Initialize ports
	clrf	PORTB
	clrf	PORTC

	clrf	TRISA		;Port A is output

	setf	TRISB		;Ports B & C are inputs
	setf	TRISC

remap
	clrf	TBLPTRU	

; clear PIC memory for variables and memory mapping

	clrf	FSR0H		; lfsr causes problems!
	clrf	FSR0L
clr1	clrf	POSTINC0
	btfss	FSR0L,6		; reached 0x40 yet?
	bra	clr1

	clrf	enromoff
	clrf	adrsource	; clears DATA/ISA address flag
;________________________________________________________________________________________
;This code between the cont-lines is for the first [ON] after module plug-in or NUT wake up, when the relocation must be done.

	bsf	locpend,0	; ROM not yet re-located, sets the corresponding flag
	bsf	INTCON2,6	; Activates INT0 on rising edge

	clrf	INTCON		; Resets INT0 bit
	bsf	INTCON,4	; Enables INTO
				; Globally disables interrupts.

	bcf	RAMPOW		; Solves bug reported by Miki Mihajlovic. Thanks!! ;-)
	bcf	RAMPOW1	
	setf	TRISA		; all of A is input (to prevent BAT from draining 1.2mA thru 10k RAM data&clk pull-up resistors)
	btfss	NUTPWO		; Stop-Watch bug reported by Geir Isene (The nastiest bug up to date... :-)
	sleep			; Waits until rising edge on PWO (PORTB,0)

	clrf	TRISA		; all of A is output (bug reported by Geir Isene... Thanks so much)

	bcf	INTCON,1	; Resets INT0 bit after PWO rises. (First Power ON)
	bcf	INTCON2,6	; Activates INT0 on falling edge
	bsf	INTCON,GIE	; Globally enables interrupts.

	bsf	rom9b1map,0	; Enables RAM at page 9
	bsf	romab1map,0	; Enables RAM at page A
	bsf	rombb1map,0	; Enables RAM at page B

;********************************* Testing RAM chips switching ON/OFF
	clrf	PORTA
	btfsc	rammap00,0
	bsf	RAMPOW
	btfss	rammap00,0
	bsf	RAMPOW1


	movlw	0x40		; Maps 1st HEPAX ROM page
	movwf	rom8b1map	; to page 8 bank 1

	movlw	0x50		; Maps 2nd HEPAX ROM page
	movwf	rom8b2map	; to page 8 bank 2

	movlw	0x60		; Maps 3rd HEPAX ROM page
	movwf	rom8b3map	; to page 8 bank 3

	movlw	0x70		; Maps 4th HEPAX ROM page
	movwf	rom8b4map	; to page 8 bank 4

#include	"mappingH.asm"

	bra	syncseek

; End of first [ON]
;________________________________________________________________________________________


start

	bsf	INTCON2,6	; Activates INT0 on rising edge

	clrf	INTCON		; Resets INT0 bit
	bsf	INTCON,4	; Enables INTO
				; Globally disables interrupts.

	clrf	enromoff	; * Important!! Prevents HEPAX bank-switch scheme to stuck on page 3 added in ver 02.

	movlw	0x03		; 3 iterations of 7 cycles (83nS/cycle) delay loop (3*7*83.33 E-9=1.75 E-6) 1.75uS total delay

synlwt	
	call	s4nop
	decfsz	WREG
	bra	synlwt		; Delay required for the Halfnut to calm down nut bus signals. (15uS)

	btfss	NUTSYNC		; If SYNC is Low then NUT is OFF, so relocation must be performed again, otherwise it's a normal SLEEP.
	bra	remap		; Go to perform relocation again
;	bcf	RAMPOW		; RAM power off
;	bcf	RAMPOW1
	clrf	TRISA
	setf	TRISA		; all of A is input (to prevent BAT from draining 1.2mA thru 10k RAM data&clk pull-up resistors)

;==========================================================================================================================================
;Fixin auto-OFF bug by allowing interrups on PORTB [7:4] change... NUTSYNC is in PORTB,5
;if NUTSYNC goes LOW while calc is on light sleep mode, is means that 10min timer has expired and Auto-OFF has occurred
;PIC should wake up and go to "begin" as if the module where just plugged in.

	movf	PORTB		; reads PORTB to avoid mismatch conditions that may cause unwanted interrupts
	bcf	INTCON,0	; clears RBIF (flag indicating that any PORTB [7:4] has chaged state)

	movlw	0x7f		; filters only bit7 (DATA).
	movwf	TRISB

	bsf	INTCON,3	; Enables PORTB interrupt on change	

	sleep			; Waits until rising edge on PWO (PORTB,0)
				; or PORTB change
	setf	TRISB		; All PORTB is input
	btfsc	NUTPWO		; If PWO is HIGH then no Auto-OFF
	bra	noauoff		; go to normal sequence

	btfsc	NUTSYNC		; Check if SYNC has gone LOW
	bra	start		; if PWO is LOW and SYNC is HIGH then some noise in the bus has waked-up the PIC
				; ignore it and go to sleep again resetting interrupt pattern
	clrf	INTCON
	bra	begin		; if POW is LOW and SYNC is LOW then Auto-OFF has occurred, go to begin to reinitialize PIC.
		
noauoff				; it'a normal keystroke sequence
	
	clrf	TRISA		; all of A is output (bug reported by Geir Isene... Thanks so much)

	bcf	INTCON,1	; Resets INT0 bit after PWO rises.
	bcf	INTCON2,6	; Activates INT0 on falling edge
	bcf	INTCON,3	; Disables interrupt on PORTB change
	bcf	INTCON,0	; Resets flag of Interrupt on PORTB change
	bsf	INTCON,GIE	; Globally enables interrupts.

;********************************* Testing RAM chips switching ON/OFF
	clrf	PORTA
	btfsc	rammap00,0
	bsf	RAMPOW
	btfss	rammap00,0
	bsf	RAMPOW1


syncseek
	wtfe	NUTSYNC		; wait for SYNC to go low

pulse0				; we are now in pulse 0
	call	swtre1

;	call	swtre2		; after routine finishes, we are at the beginning of pulse 1
		
	btfsc	NUTSYNC		; test SYNC to avoid false sync. due to old 41's pulses
	bra	syncseek	; restart syncing due to false SYNC detection.
	clrf	dtl8		; Clears data low8 for WRITE S&X
	clrf	dth2		; Clears data high2 for WRITE S&X

	call	swtre1		; 1 (data valid phase 1)
	call	swtre1		; 2 (Data valid strobe on phase 2)

	btfsc	NUTDATA
	bsf	dtl8,0		; DATA line data low, bit 0

	call	swtre1		; 3

	btfsc	NUTDATA
	bsf	dtl8,1		; DATA line data low, bit 1

	call	swtre1		; 4

	btfsc	NUTDATA
	bsf	dtl8,2		; DATA line data low, bit 2
	clrf	adrsource	; Clears RAM address selection pointer

	call	swtre1		; 5

	btfsc	NUTDATA
	bsf	dtl8,3		; DATA line data low, bit 3
	clrf	adrhg		; Clears higher address ROM/RAM read pointer

	call	swtre1		; 6

	btfsc	NUTDATA
	bsf	dtl8,4		; DATA line data low, bit 4
	clrf	addru		; Clears upper address RAM write pointer

	call	swtre1		; 7

	btfsc	NUTDATA
	bsf	dtl8,5		; DATA line data low, bit 5
	clrf	adrlw		; Clears lower address ROM/RAM read pointer

	call	swtre1		; 8

	btfsc	NUTDATA
	bsf	dtl8,6		; DATA line data low, bit 6
	clrf	addrl		; Clears lower address RAM write pointer

	call	swtre1		; 9

	btfsc	NUTDATA
	bsf	dtl8,7		; DATA line data low, bit 7

	wtre	NUTPHI1		; 10
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	dth2,0		; DATA line data high, bit 0

				; Reserved for Bankswitching
				; Test if the last opcode fetched was ENBANK1, 2, 3 or 4

	movf	oph2,W
	andlw	0x07		; bit 2 was set if SYNC was hi, indicating an opcode fetch
	sublw	0x05
	bnz	notenbank1	;-*- if it's not zero then no ENBANK opcode.
	movf	opl8,W
	andlw	0x3F
	bnz	notenbank0	;-*- if not B'xx000000 not enbank
	clrf	enromoff
	movf	opl8,W
	andlw	0xC0		;-*- filters bits 7 & 6 from the opcode
	rrncf	WREG		
	rrncf	WREG		;-*- puts 7 & 6 in 5 & 4 places
	iorwf	enromoff,1	;-*- inserts bits 5 & 4 into enromoff. End of ENBANK process

notenbank0
	movf	opl8,W		; gets the lower opcode byte
	sublw	0xF0		; checks if it's H'F0 (opcode H'1F0)
	bnz	notenbank1	; if it isn't go away
	movf	dtl8,W		; if it is get the page number 
	andlw	0x0F		; from DATA lower nibble
	movwf	FSR0L		; points to map table
	btfsc	INDF0,0		; is it RAM??
	btg	INDF0,1		; Toggle protection flag
	
notenbank1

	wtre	NUTPHI1		; 11
;-------------------------------------------------------------------------------
	btfsc	NUTDATA
	bsf	dth2,1		; DATA line data high, bit 1

				; This Phase is used for WRITE S&X (H'040) and Move ROM (H'030) detection
				; Depending on whether a Write cycle is detected or not
	movf	oph2,W		; we set adrsource,0 to 0 or 1 (ISA or DATA).
	andlw	0x07		; Bit 2 set if it was a Fetch cycle!
	sublw	0x04		; checks that U2 bits are both 0's
	bnz	adrisa		; if they're not, it isn't a WRITE s&x opcode
	movf	opl8,W		; if U2 are 0's check L8
	sublw	0x40		; if it's 0x40 we've got a WRITE s&x code
	bnz	adris0		; If it isn't get address from ISA
	bsf	adrsource,0	; WRITE CYCLE, Address from DATA 
	bra 	adrisa
adris0
	movf	opl8,W		; Checks if it's H'030
	sublw	0x30
	bnz	adrisa		; If it isn't go ahead...
	bsf	locpend,1	; if it's Mov ROM, set "Relocation OP" flag

adrisa				; No Write s&x cycle so address must be taken from ISA

	wtre	NUTPHI1		; 12
;-------------------------------------------------------------------------------
	btfss	locpend,1	; Relocation process 1st part (ROM banks 1, 2, 3 & 4)
	bra	norel1
	movf	dtl8,W		; Use the Lower 4 bits
	andlw	0x0F
	movwf	FSR0L		; as a pointer to the map table
	movlw	0x40		;	
	movwf	INDF0		;

	bsf	FSR0L,4
	movlw	0x60
	movwf	INDF0

	bsf	FSR0L,5			
	movlw	0x70
	movwf	INDF0

	bcf	FSR0L,4
	movlw	0x50
	movwf	INDF0

norel1

	wtre	NUTPHI1		; 13
;-------------------------------------------------------------------------------
	btfss	locpend,1	; Relocation process 2nd part (RAM recovery)
	bra	norel2
	movf	rom9b1map,w
	movwf	rom8b1map
	clrf	rom8b2map
	clrf	rom8b3map
	clrf	rom8b4map
	clrf	locpend

norel2 

#include	"addrs08m.asm"

;-------------------------------------------------------------------------------

ramrd
	bsf	RAMDAT		; Sends START sequence again
	nop
	nop
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			
	nop			
	bcf	RAMDAT		
	nop
	nop
	bcf	RAMCLK

	wthi	NUTPHI1		; 31
;-------------------------------------------------------------------------------
	btfsc	NUTISA
	bsf	addru,7		; ISA line address high, bit 7

	movf	addru,W
	andlw	0xF0		; use the upper four bits ...
	swapf	WREG
	movwf	FSR0L		; as a pointer to the map table
	movf	INDF0,W
	btfsc	WREG,0		; If map is B'XXXXXXX1 then is RAM
	bra	ouram		; Skips the STOP sequence

				; Sends STOP sequence

	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			
	nop
	bsf	RAMDAT		; STOP sequence
	nop
	nop
	bcf	RAMCLK
	nop
	nop
	bcf	RAMDAT
	bra	checkrom	; We've verified that it's not our RAM 
				; but it can still be our ROM... go check it!

ouram				; Read sequence

;So it's our RAM after all, good let's trate those bits... ;-)
;Phi# |    32   |    33   |    34   |    35   |    36   |    37   |    38   |    39   |    40   |    41   |    42   |    43   |    44   |    45   |

;CLK __/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__/\__

;DAT _/\_____/\____________________/\_____XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX_____XXXXXXXXXX______________________________/\__/\_
;       ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^    ^     ^
;       1    0    1    0    0    0    0    1   ACK  D0   D1   D2   D3   D4   D5   D6   D7   ACK  D8   D9   D10  D11  D12  D13  D14  D15 NO-ACK STOP
;      |----SLAVE ID----|  |-DEVICE AD-| READ

	wtre	NUTPHI1		; 32
;-------------------------------------------------------------------------------
	bsf	RAMDAT		; SDT to "1" (Start slave address "B'10100001")
	call	ferclk
	nop
;------------------- 
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100001" 2nd bit)
	call	ferclk

	wthi	NUTPHI1		; 33
;-------------------------------------------------------------------------------
	bsf	RAMDAT		; SDT to "1" (Slave address "B'10100001" 3rd bit)
	call	ferclk
	nop
;------------------- 
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100001" 4th bit)
	call	ferclk

	wthi	NUTPHI1		; 34
;-------------------------------------------------------------------------------
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100001" 5th bit)
	call	ferclk
	nop
;------------------- 
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100001" 6th bit)
	call	ferclk

	wthi	NUTPHI1		; 35
;-------------------------------------------------------------------------------
	bcf	RAMDAT		; SDT to "0" (Slave address "B'10100001" 7th bit)
	call	ferclk
	nop
;------------------- 
	bsf	RAMDAT		; SDT to "1" (Slave address "B'10100001" 8th bit) 
	call	ferclk
	bcf	RAMDAT		; Stabilizes DATA read line... 21/04/2010!!!
	wthi	NUTPHI1		; 36
;-------------------------------------------------------------------------------
	bsf	DATDIR		; Set RAMDAT as Input for ACK and Data
	call	ferclk
	clrf	opl8
;	clrf	oph2		; Bug detected by Gene. Thaks a lot!!
	bcf	oph2,0		; This prevents FETCH cycle flag (oph2,2)
	bcf	oph2,1		; from being erroneously erased... with painfull consecuences!!
;------------------- 
	call	s4nop
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	opl8,0		; 3
	nop			; 4
	bcf	RAMCLK		; and 5, go down...
	call	s4nop

	wthi	NUTPHI1		; 37
;-------------------------------------------------------------------------------
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	opl8,1		; 3
	nop			; 4
	bcf	RAMCLK		; and 5, go down...
	call	s6nop
;------------------- 
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	opl8,2		; 3
	nop			; 4
	bcf	RAMCLK		; and 5, go down...
	call	s6nop
;------------------- 
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)

	wthi	NUTPHI1		; 38
;-------------------------------------------------------------------------------
	btfsc	RAMDAT		; 3
	bsf	opl8,3		; 4
	bcf	RAMCLK		; and 5, go down...
	call	s6nop
;------------------- 
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	opl8,4		; 3
	nop			; 4
	bcf	RAMCLK		; and 5, go down...
	call	s6nop
;------------------- 
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	opl8,5		; 3

	wthi	NUTPHI1		; 39
;-------------------------------------------------------------------------------
	bcf	RAMCLK		; and 5, go down...
	call	s6nop
;------------------- 
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	opl8,6		; 3
	nop			; 4
	bcf	RAMCLK		; and 5, go down...
	call	s6nop
;------------------- 
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	opl8,7		; 3
	nop			; 4
	bcf	RAMCLK		; and 5, go down...

	wthi	NUTPHI1		; 40
;-------------------------------------------------------------------------------
	bcf	RAMDAT		; Acknowledge: DAT = "0"
	bcf	DATDIR		; Set RAMDAT as Output for ACK
	call	ferclk
	bsf	DATDIR		; Set RAMDAT as Input again.
	call	s6nop
;------------------- 
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	oph2,0		; 3 bit 0
	nop			; 4
	bcf	RAMCLK		; and 5, go down...
	nop
	nop

	wthi	NUTPHI1		; 41
;-------------------------------------------------------------------------------
	nop			;
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	btfsc	RAMDAT		; 2
	bsf	oph2,1		; 3 bit 1
	nop			; 4
	bcf	RAMCLK		; and 5, go down...
	call	s4nop		; Data has been read no further RAM activity required
;------------------- 
	call	ferclk
	call	s4nop
;------------------- 
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)

	wthi	NUTPHI1		; 42
;-------------------------------------------------------------------------------
	bcf	RAMCLK		; and 5, go down...
	call	s4nop
;------------------- 
	call	ferclk
	nop
	nop
;------------------- 
	call	ferclk

	wthi	NUTPHI1		; 43
;-------------------------------------------------------------------------------
	bsf	RAMDAT		; NOT Acknowledge: DAT = "1"
	bcf	DATDIR		; Set RAMDAT as Output for NO-ACK
	call	ferclk
	bcf	RAMDAT

	wthi	NUTPHI1		; 44
;-------------------------------------------------------------------------------

	wtre	NUTPHI2		; 45
;-------------------------------------------------------------------------------
	bra	outfromisa

;****************************************************************************************************************************

keeptrack
	wtre	NUTPHI1		; 31

checkrom
	wtre	NUTPHI1		; 32

	call	swtre1		; 33

	call	swtre1		; 34

	call	swtre1		; 35

	call	swtre1		; 36

	call	swtre1		; 37

	call	swtre2		; 38

	movf	addru,W		; Gets address upper byte
	sublw	0x41		; Is it H'41?
	bnz	check41map	; If it ist'n go to checkrom
	movf	addrl,W		; Gets address lower byte
	bnz	check41map	; If it ist'n go to checkrom	
	movf	rammap00,W	; Gets RAM map byte
	movwf	opl8		; place it as code for output on ISA line
;	clrf	oph2
	movf	rammap01,w
	movwf	oph2

	call	swtre2		; 39

	call	swtre2		; 40

	call	swtre2		; 41

	call	swtre2		; 42

	call	swtre2		; 43

	call	swtre2		; 44

	call	swtre2		; 45
	bra	outfromisa
	
check41map
	call	swtre2		; 39

	call	swtre2		; 40

	call	swtre2		; 41
endwrite
	wtre	NUTPHI2		; 42

;; The address of the opcode to fetch is in 
;; We compute where in the table to find oph2 and opl8
;; get the upper four bits of the address, and see where
;; it maps in our tables.

	movf	addru,W
	andlw	0xF0		; use the upper four bits ...
	swapf	WREG
	addwf	enromoff,W
	movwf	FSR0L		; as a pointer to the map table
	movf	INDF0,W
	btfsc	WREG,0
	bsf	adrsource,1	; RAM has been WRITTEN and intended to be READ!!
				; Send NOP!!!! 
	andlw	0xF0		; only the upper nibble is usefull for ROM.
	bnz	ours
	movlw	0x08
	goto	notours		; eat the next 14 pulses, saving the op code
	
ours

	call	swtre2		; 43

	movwf	TBLPTRH
	movf	addru,W
	andlw	0x0f
	iorwf	TBLPTRH,F
	movf	addrl,W
	movwf	TBLPTRL
	tblrd	*
	movff	TABLAT,opl8

	rrcf	TBLPTRH
	rrcf	TBLPTRL
	rrcf	TBLPTRH
	rrcf	TBLPTRL
	movf	TBLPTRH,W
	andlw	0x1f
	movwf	TBLPTRH
	tblrd	*
	movff	TABLAT,oph2


	call	swtre2		; 44


	btfss	addrl,1
	bra	norot4
	rrcf	oph2
	rrcf	oph2
	rrcf	oph2
	rrcf	oph2
norot4	
	btfss	addrl,0
	bra	norot2
	rrcf	oph2
	rrcf	oph2
norot2

	call	swtre2		; 45

outfromisa
	movlw	0x03
	andwf	oph2,F		; good time to mask this
	movlw	0x42		; Points to opl8
	movwf	FSR0L

;	wtlo	NUTPHI2		; 45; start Word output (ISA)

	call	OUTL8H2		; 46

	call	OUTL8H2		; 47

	call	OUTL8H2		; 48

	call	OUTL8H2		; 49

	call	OUTL8H2		; 50

	call	OUTL8H2		; 51

	call	OUTL8H2		; 52

	call	OUTL8H2		; 53

	incf	FSR0L		; Points to oph2

	call	OUTL8H2		; 54

	btfsc	NUTSYNC		; are we an opcode fetch?
	bsf	oph2,1		; indicate that (this does not affect ISA output)

	call	OUTL8H2		; 55

	rlncf	oph2		; Puts oph2 bits back in place.
	rlncf	oph2

	wtre	NUTPHI2		; 0

;	bcf	NUTISA		; prevent a glitch?
	setf	TRISB

	goto	pulse0
notours	
	call	swtre2		; 43

	btfsc	adrsource,1
	bra	sendnop
	call	swtre2		; 44
;	clrf	opl8
	call	swtre1		; 44
;	clrf	oph2
	call	swtre1		; 45
	
keepeating
	call	swtre1		; 46 - 53
;	btfsc	NUTISA
;	bsf	opl8,7
;	rrncf	opl8
	decfsz	WREG
	bra	keepeating
;	btfsc	NUTSYNC
;	bsf	oph2,2
	call	swtre1		; 54
;	btfsc	NUTISA
;	bsf	oph2,0
	call	swtre1		; 55
;	btfsc	NUTISA
;	bsf	oph2,1
	goto	pulse0

sendnop
	call	swtre2		; 44
	clrf	opl8
	clrf	oph2
	bra	norot2
	
;------------------------------ Subroutines area
swtre1	
	btfsc	NUTPHI1
	bra	swtre1
lo1	btfss	NUTPHI1
	bra	lo1
	return

swtre2	
	btfsc	NUTPHI2
	bra	swtre2
lo2	btfss	NUTPHI2
	bra	lo2
	return	

ferclk
	bsf	RAMCLK		; SCL to "1" (Must be kept for 5 cycles)
	nop			; 1
	nop			; 2
	nop			; 3
	nop			; 4
	bcf	RAMCLK		; and 5, go down...
	return



s6nop
	nop
s5nop
	nop
s4nop
	return

;------------------------------
OUTL8H2
	wtre	NUTPHI2		; Sincronizes with Phi2 rising edge
	bcf	NUTISA		; Bus pre-charge LOW > Activated for testing 06 Apr. 2011 Geir detected bug.
	bcf	OUTISA		; To avoid noise     > with ARCLIP (NoVCHAP) and Zenrom.

	bsf	OUTISA		; Make sure ISA is input again
	bsf	NUTISA		; ISA line = High

	call	s5nop		; Required?
	call	s5nop

	btfsc	INDF0,0		; Check bit to send if 0 then do nothing
	bcf	OUTISA		; If bit =  1 then ISA is output
	rrncf	INDF0		; Shifts to the next bit
;	wtfe	NUTPHI1		; Sincronizes with Phi1 falling edge (HP41 read)
;	call	s5nop
;	bsf	OUTISA		; Make sure ISA is input again.
	return


; ------------------- ROM images loading zone

#include	"HEPX1d-0.asm"
#include	"HEPX1d-1.asm"
#include	"HEPX1d-2.asm"
#include	"HEPX1d-3.asm"

#include	"romimg0H.asm"
#include	"romimg1H.asm"


		END	; directive 'end of program'
